.yoast-link-suggestion-icon.yoast-link-suggestion-icon{box-sizing:border-box}.yoast-link-suggestions--loading{width:75px;height:75px;display:block;margin:0 auto;padding:3em 0}#yoast_internal_linking .inside{margin-top:0;padding-top:10px;padding-bottom:25px}.yoast-links-suggestions-notice{margin:5px 0 2px}.yoast-redirect-notification-modal-url{margin:0}.yoast-redirect-notification-modal-buttons{display:flex;align-items:center;margin-top:16px}.yoast-redirect-notification-modal .yoast-redirect-notification-modal-button{margin-left:16px}#wpseosynonyms{margin-bottom:2em}.yoast-section{box-shadow:0 1px 2px rgba(0,0,0,.2);position:relative;background-color:#fff;padding:0 20px 15px}.yoast-section__heading{padding:8px 20px;font-size:.9rem;margin:0 -20px 15px;font-family:Open Sans,sans-serif;font-weight:300;color:#555}.yoast-section__heading-icon{padding-right:44px;background-repeat:no-repeat;background-position:right 20px top .6em;background-size:16px}.yoast-section,.yoast-section *,.yoast-section:after,.yoast-section :after,.yoast-section:before,.yoast-section :before{box-sizing:border-box}.editable-preview h3:first-child{margin-top:0}.wpseo-form .snippet-editor__label{display:inline-block;width:auto;margin-top:24px}.wpseo-form .snippet-editor__label:first-child{margin-top:0}.wpseo-form .yoast_help.yoast-help-button{margin-top:20px}.wpseo-form .snippet-editor__label:first-child+.yoast-help-button{margin-top:-4px}#yoast-insights{margin-top:2em}.remove-keyword{display:inline-block;margin:0 -3px 0 0;padding:5px 6px 6px;border:none;background:transparent;cursor:pointer}.remove-keyword span{display:inline-block;width:16px;height:16px;padding:0;border:1px solid transparent;border-radius:100%;color:#686868;font-size:12px;font-weight:400;line-height:15px;text-align:center;vertical-align:top}.remove-keyword:focus{outline:none;box-shadow:0 0 0 1px #5b9dd9,0 0 2px 1px rgba(30,140,190,.8)}.remove-keyword:focus span,.remove-keyword:hover span{color:#fff;background-color:#dc3232}.wpseo-add-keyword:disabled{display:none}.wpseo_content_tab+.wpseo_keyword_tab .wpseo-keyword{font-weight:600}.wpseo_content_tab+.wpseo_keyword_tab a[data-score=na] .wpseo-keyword{font-weight:400}.wpseo_keyword_tab a[data-score=na] .wpseo-keyword{max-width:1em;max-width:1rem}.wpseo_keyword_tab.active a[data-score=na] .wpseo-keyword{max-width:none}.wpseo-metabox-tabs .wpseo_keyword_tab_hideable .wpseo_tablink{padding-left:0}.wrap .wpseo-notice-breakout-inside{margin:0 -12px}